#include "ehs/io/hid/ButtonState.h"

namespace ehs
{
	ButtonState::ButtonState()
		: state(State::RELEASED), pressed(false), threshold(1.0f)
	{
	}

	ButtonState::ButtonState(const Button& button, const State state)
		: button(button), state(state), pressed(true), threshold(1.0f)
	{
	}

	ButtonState::ButtonState(const ButtonState& bs)
		: button(bs.button), state(bs.state), pressed(bs.pressed), threshold(bs.threshold)
	{
	}

	ButtonState& ButtonState::operator=(const ButtonState& bs)
	{
		if (this == &bs)
			return *this;

		button = bs.button;
		state = bs.state;
		pressed = bs.pressed;
		threshold = bs.threshold;

		return *this;
	}

	bool ButtonState::operator==(const Button& other) const
	{
		return button == other;
	}

	bool ButtonState::operator!=(const Button& other) const
	{
		return button != other;
	}

	bool ButtonState::operator==(const State otherState) const
	{
		return state == otherState;
	}

	bool ButtonState::operator!=(const State otherState) const
	{
		return state != otherState;
	}

	Button ButtonState::GetButton() const
	{
		return button;
	}

	void ButtonState::SetState(State newState)
	{
		state = newState;
	}

	State ButtonState::GetState() const
	{
		return state;
	}

	void ButtonState::SetPressed(bool value)
	{
		pressed = value;
	}

	bool ButtonState::IsPressed() const
	{
		return pressed;
	}

	void ButtonState::SetThreshold(const float newThreshold)
	{
		threshold = newThreshold;
	}

	float ButtonState::GetThreshold() const
	{
		return 1.0f;
	}
}